/**
 * @author bcoxie
 */

var mWidth = 800;
var mHeight = 600;
var mFps = 60;
var mMillisPerFrame = 1000 / mFps;
var mCanvas = document.getElementById('gameCanvas');
var mContext = mCanvas.getContext('2d');
var gLoop;
var mPrevTime;
var mFpsCounter;
var mCurrFps;
var mPrevFrameTime;
var mTD;
var mGroundHeight = mHeight - 100;
var mGravity = 3000;
var mDeathFireWidth = 50;

var mPlayer;
var mBlocks = [];
var numBlocks = 10;
var mDeathFire;
var mGameOver = false;
var bGameHasRestarted = false;
var mBackGround;

var mBat;
var mBatSlowDown = 0;
var mBatSlowEffect = 0.5;

var mScore = 0;
var mHighScore = 0;
//var block;

mCanvas.width = mWidth;
mCanvas.height = mHeight;

//document.onkeypress = "handleKeyPress(evt)";

// var img = new Image();
// img.src = "images/background.png";
// mBackGround= new Sprite(img, 0, 0, 512, 512);

initializeGame();
gameLoop();

function gameLoop()
{
	//update(mMillisPerFrame);
	//draw();
	
	//gLoop = setTimeout(gameLoop, mMillisPerFrame);
	mFpsCounter++;
	
	var currTime = new Date();
	var deltaTime = currTime - mPrevTime;
	//var capFps = false;
	
	if (currTime - mPrevFrameTime >= 1000)
	{
		mCurrFps = mFpsCounter;
		mFpsCounter = 0;
		mPrevFrameTime = currTime;
	}
	
	//if (deltaTime < mMillisPerFrame)
	//{
		//gLoop = setTimeout(gameLoop, mMillisPerFrame - deltaTime);
		//capFps = true;
	//}
	
	mPrevTime = currTime;
	
	update(deltaTime / 1000);
	draw();
	
	//gLoop = setTimeout(gameLoop, mMillisPerFrame - deltaTime);
	gLoop = setTimeout(gameLoop, mMillisPerFrame);
}

function initializeGame()
{
	mPlayer = new Player();
	mPrevTime = new Date();
	mPrevFrameTime = mPrevTime;
	mFpsCounter = 0;
	mCurrFps = 0;
	mScore = 0;
	
	//block = new Block();
	createBlocks();
	
	mDeathFire = new DeathFire();
	mBackGround = new BackGround();
	mBat = new Bat();
}

function restartGame()
{
	bGameHasRestarted = true;
	mGameOver = false;
	
	if (mScore > mHighScore)
	{
		mHighScore = mScore;
	}
	
	initializeGame();
}

function update(deltaTime)
{
	if (!mGameOver)
	{
		mPlayer.update(deltaTime);
		mBat.update(deltaTime);
		
		for (var i = 0; i < numBlocks; i++)
		{
			mBlocks[i].update(deltaTime);
		}
		
		mScore += deltaTime;
		
		mBatSlowDown -= deltaTime;
	}
}

function draw()
{
	clear();
	
	//mBackGround.draw(mContext, 0, 0, 200, 200);
	mBackGround.draw(mContext);
	
	drawGround();
	//block.draw(mContext);
	for (var i = 0; i < numBlocks; i++)
	{
		mBlocks[i].draw(mContext);
	}
	
	mBat.draw(mContext);
	
	mPlayer.draw(mContext);
	mDeathFire.draw(mContext);
	//mBackGround.draw(mContext);
	
	drawFps();
	
	if (mGameOver)
	{
		drawGameOver();
	}
}

function drawFps()
{
	mContext.lineWidth=1;
	mContext.fillStyle="#CC00FF";
	mContext.lineStyle="#ffff00";
	mContext.font="18px sans-serif";
	mContext.fillText(mCurrFps, 20, 20);
	//mContext.fillText(mTD, 80, 60);
	
	mContext.font="30px sans-serif";
	mContext.fillText("Score: " + Math.floor(mScore), 80, 30);
	mContext.fillText("High Score: " + Math.floor(mHighScore), 500, 30);
	
	mContext.font="20px sans-serif";
	mContext.fillText("ASWD plus Arrow Keys", 300, 550);
}

function drawGameOver()
{
	mContext.lineWidth=1;
	mContext.fillStyle="#CC00FF";
	mContext.lineStyle="#ffff00";
	mContext.font="64px sans-serif";
	//mContext.fillText(mCurrFps, 20, 20);
	//mContext.fillText(mTD, 80, 20);
	mContext.fillText("Game Over", 250, mHeight / 2);
	
	mContext.fillStyle="#FBB117";
	//mContext.lineStyle="#ffff00";
	mContext.font="24px sans-serif";
	//mContext.fillText(mCurrFps, 20, 20);
	//mContext.fillText(mTD, 80, 20);
	mContext.fillText("Press X to restart", 320, mHeight / 2 + 30);
}

function drawGround()
{
	//mContext.fillStyle = '#646D7E';
	mContext.fillStyle = '#463E41';
	mContext.beginPath();
	mContext.rect(0, mGroundHeight, mWidth, mHeight - mGroundHeight);
	mContext.closePath();
	mContext.fill();
}

function clear()
{
	mContext.fillStyle = '#00FFFF';
	mContext.beginPath();
	mContext.rect(0, 0, mWidth, mHeight);
	mContext.closePath();
	mContext.fill();
}

function createBlocks()
{
	if (bGameHasRestarted)
	{
		for (var i = 0; i < numBlocks; i++)
		{
			//mBlocks[i] = new Block();
			mBlocks[i].x = Math.random() * mWidth;
			mBlocks[i].y = Math.random() * (mGroundHeight - mBlocks[i].height);
			mBlocks[i].hp = mBlocks[i].maxHp;
		}
	}
	else
	{
		for (var i = 0; i < numBlocks; i++)
		{
			mBlocks.push(new Block());
		}
	}
}

