/**
 * @author bcoxie
 */

function Player()
{
	this.x = 400;
	this.y = 400;
	this.width = 50;
	this.height = 100;
	this.velocityX = 0;
	this.velocityY = 0;
	this.accelX = 0;
	this.accelY = 0;
	this.right = false;
	this.left = false;
	this.up = false;
	this.down = false;
	this.bJumping = false;
	this.dash = false;
	this.dashRight = false;
	this.dashLeft = false;
	this.dashUp = false;
	this.dashDown = false;
	this.dashSpeed = 2000;
	this.moveSpeed = 500;
	this.gravityDelay = 0;
	this.gravityDelayAmt = 0.0;
	this.gravityY = mGravity;
	this.speedDrag = 0.85;
	this.jumpForce = 1500;
	this.canMoveDown = true;
	this.canMoveUp = true;
	this.canMoveRight = true;
	this.canMoveLeft = true;
	this.dashDelay = 0;
	this.dashDelayAmt = 0.2;
	this.startDashX = 0;
	this.startDashY = 0;
	this.canDash = true;
	this.dashTime = 0.2;
	this.dashingTime = 0;
	this.dashing = false;
	this.maxDashParticles = 20;
	this.dashParticles = [];
	this.currDashParticleIndex = 0;
	this.dashBounceForce = 1000;
	this.bounceForceX = 0;
	this.bounceForceY = 0;
	//this.aParticle;
	
	var img = new Image();
	img.src = "images/princess.png";
	this.sprite= new Sprite(img, 0, 0, 128, 256);
	
	var dashImage = new Image();
	dashImage.src = "images/dashright.png";
	this.dashSprite = new Sprite(dashImage, 0, 0, 512, 128);
	
	for (var i = 0; i < this.maxDashParticles; i++)
	{
		this.dashParticles.push(new DashParticle());
	}
	
	//this.aParticle = new DashParticle();
}

Player.prototype.draw = function(context)
{
	//if (this.dash)
	if (this.dashDelay > 0)
	{
		//this.dashSprite.draw(context, this.x, this.y, 512, 128);
		var startX;
		var startY;
		var endX;
		var endY;
		
		if (this.x > this.dashStartX)
		{
			// going right
			startX = this.dashStartX;
			endX = this.x;
		}
		else
		{
			// going left
			startX = this.x;
			endX = this.dashStartX;
		}
		
		if (this.y > this.dashStartY)
		{
			// going down
			startY = this.dashStartY;
			endY = this.y;
		}
		else
		{
			// going up
			startY = this.y;
			endY = this.dashStartY;
		}
		
		startY = startY + this.height / 4;
		endyY = endY + this.height / 4;
		//this.dashSprite.draw(context, this.dashStartX, this.dashStartY, 
			//this.x - this.dashStartX, this.y - this.dashStartY);
			
		//this.dashSprite.draw(context, startX, startY, 
			//endX - startX + (this.width / 2), 50);//endY - startY);
			
		this.dashParticles[this.currDashParticleIndex].spawn(this.x + this.width / 2, 
			this.y + this.height / 2);
		this.currDashParticleIndex++;
		
		if  (this.currDashParticleIndex >= this.maxDashParticles)
		{
			this.currDashParticleIndex = 0;
		}
	}
	//else
	//{
		this.sprite.draw(context, this.x, this.y, this.width, this.height);
	//}
	
	for (var i = 0; i < this.maxDashParticles; i++)
	{
		this.dashParticles[i].draw(context);
	}
	
	//this.aParticle.spawn(this.x, this.y);
	//this.aParticle.draw(context);
}

Player.prototype.update = function(deltaTime)
{
	if (this.dashDelay > 0)
	{
		this.dash = false;
		this.canDash = false;
		this.dashDelay -= deltaTime;
	}
	// movement
	if (this.dash)
	{
		this.dashMove(deltaTime);
	}
	else
	{
		this.regularMove(deltaTime);
	}
	
	// gravity
	if (this.gravityDelay <= 0)
	{
		this.gravityY = mGravity;
	}
	else
	{
		this.gravityY = 0;
		this.gravityDelay -= deltaTime;
	}
	
	// drag
	if (this.velocityX != 0)
	{
		this.velocityX *= this.speedDrag;
	}
	if (this.velocityY != 0)
	{
		this.velocityY *= this.speedDrag;
	}
	
	var isDashing = false;
	if (this.dashDelay > 0)
	{
		isDashing = true;
	}
	
	this.canMoveDown = true;
	this.canMoveUp = true;
	this.canMoveRight = true;
	this.canMoveLeft = true;
	this.checkBatCollision();
	this.checkCollisions();
	this.screenBounds();
	
	
	if (isDashing && !this.canMoveDown)
	{
		this.velocityY = -this.dashBounceForce;
	}
	if (isDashing && !this.canMoveUp)
	{
		this.velocityY = this.dashBounceForce;
	}
	if (isDashing && !this.canMoveRight)
	{
		this.velocityX = -this.dashBounceForce;
	}
	if (isDashing && !this.canMoveLeft)
	{
		this.velocityX = this.dashBounceForce;
	}
	
	for (var i = 0; i < this.maxDashParticles; i++)
	{
		this.dashParticles[i].update(deltaTime);
	}
	
	// check game over
	if (this.x < mDeathFireWidth)
	{
		mGameOver = true;
	}
}

Player.prototype.checkBatCollision = function()
{
	if (rectIntersects(this, mBat))
	{
		if (mBat.hp > 0)
		{
			if (this.dashDelay > 0)
			{
				mBat.hp -= 1;
			}
			else
			{
				
			}
		}
	}
}

Player.prototype.checkCollisions = function()
{
	//this.height = 100;
	var centerX = this.x + this.width / 2;
	var centerY = this.y + this.height / 2;
	// this.canMoveDown = true;
	// this.canMoveUp = true;
	// this.canMoveRight = true;
	// this.canMoveLeft = true;
	
	for (var i = 0; i < numBlocks; i++)
	{
		if (rectIntersects(this, mBlocks[i]))
		{
			//this.height = 50;
			// if (mBlocks[i].x > this.x)
			// {
				// this.x = mBlocks[i].x - this.width;
			// }
			// if (mBlocks[i].x < this.x)
			// {
				// this.x = mBlocks[i].x + mBlocks[i].width;
			// }
			
			if (mBlocks[i].hp > 0)
			{
				if (this.dashDelay > 0)
				{
					mBlocks[i].hp -= 1;
					//this.dashDelay = 0;
				}
			
				var otherCenterX = mBlocks[i].x + mBlocks[i].width / 2;
				var otherCenterY = mBlocks[i].y + mBlocks[i].height / 2;
				
				var xDiff = centerX - otherCenterX;
				var yDiff = centerY - otherCenterY;
				
				xDiff = xDiff / ((mBlocks[i].width + this.width) * 0.5);
				yDiff = yDiff / ((mBlocks[i].height + this.height) * 0.5);
				
				if (Math.abs(xDiff) > Math.abs(yDiff))
				{
					if (xDiff < 0)
					{
						this.x = mBlocks[i].x - this.width;
						this.canMoveRight = false;
					}
					else
					{
						this.x = mBlocks[i].x + mBlocks[i].width;
						this.canMoveLeft = false;
					}
				}
				else
				{
					if (yDiff < 0)
					{
						this.y = mBlocks[i].y - this.height;
						this.canMoveDown = false;
						this.bJumping = false;
					}
					else
					{
						this.y = mBlocks[i].y + mBlocks[i].height;
						this.canMoveUp = false;
					}
				}
			
			}
			
			//var dist = xDiff * xDiff + yDiff * yDiff;
			//this.x += xDiff * 0.2;
			//this.y += yDiff * 0.2;
			
		}
	}
}

Player.prototype.screenBounds = function()
{
	if (this.x < 0)
	{
		this.x = 0;
		this.canMoveLeft = false;
	}
	
	if (this.x + this.width > mWidth)
	{
		this.x = mWidth - this.width;
		this.canMoveRight = false;
	}
	
	if (this.y < 0)
	{
		this.y = 0;
		this.canMoveUp = false;
	}
	
	if (this.y + this.height > mGroundHeight)
	{
		this.y = mGroundHeight - this.height;
		this.velocityY = 0;
		this.bJumping = false;
		this.canMoveDown = false;
	}
};

Player.prototype.regularMove = function(deltaTime)
{
	//var dashFactor = 1;
	var moveVelX = 0;
	var moveVelY = 0;
	
	if (this.right)
	{
		//moveVelX += 300 * dashFactor;
		moveVelX += this.moveSpeed;
	}
	if (this.left)
	{
		//moveVelX += -300 * dashFactor;
		moveVelX -= this.moveSpeed;
	}

	
	if (this.down)
	{
		//moveVelY += 300 * dashFactor;
		moveVelY += this.moveSpeed;
	}
	if (this.up)
	{
		//moveVelY += -300;
		// if (this.dash)
		// {
			// //moveVelY += -300 * dashFactor;
			// moveVelY -= this.moveSpeed;
		// }
		//else if (!this.bJumping)
		if (!this.bJumping)
		{
			this.bJumping = true;
			//this.velocityY += -800;
			this.velocityY += -this.jumpForce;
		}
	}
	
	this.velocityX += this.accelX * deltaTime;
	this.velocityY += (this.accelY + this.gravityY) * deltaTime;
	
	var newX = this.x + (this.velocityX + moveVelX) * deltaTime;
	var newY = this.y + (this.velocityY + moveVelY) * deltaTime;
	//this.x += (this.velocityX + moveVelX) * deltaTime;
	//this.y += (this.velocityY + moveVelY) * deltaTime;
	if ((newX > this.x && this.canMoveRight) ||
		(newX < this.x && this.canMoveLeft))
	{
		this.x = newX;
	}
	if ((newY > this.y && this.canMoveDown) ||
		(newY < this.y && this.canMoveUp))
	{
		this.y = newY;
	}
};

Player.prototype.dashMove = function(deltaTime)
{
	var dashVelX = 0;
	var dashVelY = 0;
	
	if (this.dashRight)
	{
		//dashVelX= 1500;
		dashVelX = this.dashSpeed;
		//dashVelX = this.dashDist;
	}
	if (this.dashLeft)
	{
		//dashVelX = -1500;
		dashVelX = -this.dashSpeed;
	}
	if (this.dashDown)
	{
		//dashVelY = 1500;
		dashVelY = this.dashSpeed;
	}
	if (this.dashUp)
	{
		//dashVelY = -1500;
		dashVelY = -this.dashSpeed;
	}
	
	//this.x += dashVelX * deltaTime;
	//this.y += dashVelY * deltaTime;
	//this.x += (this.velocityX + 0) * deltaTime;
	//this.y += (this.velocityY + 0) * deltaTime;
	//this.x += dashVelX;
	//this.y += dashVelY;
	this.velocityX = dashVelX;
	this.velocityY = dashVelY;
	this.gravityDelay = this.gravityDelayAmt;
	//this.dash = false;
	
	var newX = this.x + dashVelX * deltaTime;
	var newY = this.y + dashVelY * deltaTime;
	
	if ((newX > this.x && this.canMoveRight) ||
		(newX < this.x && this.canMoveLeft))
	{
		this.x = newX;
	}
	if ((newY > this.y && this.canMoveDown) ||
		(newY < this.y && this.canMoveUp))
	{
		this.y = newY;
	}
	
	this.dashDelay = this.dashDelayAmt;
	
	//this.dash = false;
	// this.dashRight = false;
	// this.dashLeft = false;
	// this.dashDown = false;
	// this.dashUp = false;
};
